/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.accumulator;
import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.gasp.State;


/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author mh193489
 *
 * Accumulator ship. The input is wrapped in the Payload.
 * Accepts integer on In terminal and outputs the cumulative sum on Out terminal.
 */
public class Accumulator extends Component {
		
	InputTerminal in;
	OutputTerminal ackIn;
	
	OutputTerminal out;
	InputTerminal ackOut;

    OutputTerminal internalOut;
	InputTerminal ackInternalOut;
        
        OutputTerminal ackInternalIn;
        InputTerminal internalIn;
	        

                
	/** accumulator action hidden in the ship */
	AccumulatingAction accumulatorIn;
	AckAction accumulatorAck;
	
    
	public Accumulator(String name, Parameters params) {
		super(name, null);
		
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				// ignore all parametters
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");							
			} // for
		}
		
		construct();
			
	} // Accumulator

	
	public Accumulator(String name) {
		super(name, null);
		construct();
				
	} // Accumulator
	
	private void construct() {
            		
		in= new InputTerminal(AccumulatorDefaults.IN, this);
		ackIn= new OutputTerminal(AccumulatorDefaults.ACK_IN, this);

                internalIn= new InputTerminal(AccumulatorDefaults.INTERNAL_IN, this);
		ackInternalIn= new OutputTerminal(AccumulatorDefaults.ACK_INTERNAL_IN, this);
                ackInternalOut= new InputTerminal(AccumulatorDefaults.ACK_INTERNAL_OUT, this);
		internalOut= new OutputTerminal(AccumulatorDefaults.INTERNAL_OUT, this);
                
		out= new OutputTerminal(AccumulatorDefaults.OUT, this);
		ackOut= new InputTerminal(AccumulatorDefaults.ACK_OUT, this);
		
		accumulatorIn= new AccumulatingAction("ACCUMULATING_ACTION");
		accumulatorIn.attachIn(in, ackIn);
		accumulatorIn.attachOut(out, ackOut, AccumulatingAction.BRANCH_OUT);
		accumulatorIn.attachOut(internalOut, ackInternalOut, AccumulatingAction.BRANCH_ACK);
                
        accumulatorAck=  new AckAction("AckAction");
        accumulatorAck.attach(internalIn);
        accumulatorAck.attach(ackInternalIn);
            
                // create the states connecting the two actions
		State internalState= new State(AccumulatorDefaults.INTERNAL_STATE, this);
		State ackInternalState= new State(AccumulatorDefaults.ACK_INTERNAL_STATE, this);
                
        // make connections
        internalState.attach(internalIn);
        ackInternalState.attach(ackInternalIn);
        internalState.attach(internalOut);
        ackInternalState.attach(ackInternalOut);
        
        // initialize states
        ackInternalState.setInitiallyActive(null);
                
        }
 
	public InputTerminal getIn() {
		return in;
	}
	
	public OutputTerminal getAckIn() {
		return ackIn;
	}
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
	
	/** 
	 * Get a terminal by name.
	 * Return null if no such terminal exists.
	 */
	public Terminal getTerminal(String name) {
		if (name.equals(in.getBaseName()) 
				|| name.equals(in.getBaseAlias()))
			return in;
		else if (name.equals(ackIn.getBaseName()) 
				|| name.equals(ackIn.getBaseAlias()))
			return ackIn;
		else if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else 
			return null;
			
	} // getTerminal
	
} // TestShip;


